### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: readattribute.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"
 
from zope.interface import Interface,implements
from zope.filerepresentation.interfaces import IReadFile

class Attribute2ReadFileAdapter(object) :
    implements(IReadFile)
    
    def __init__(self,context) :
        self.context = context
        
    def read(self) :
        return self.context.read()
        
    def size(self) :
        return len(self.read())




### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: ftpwidget.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"

from zope.interface import implements
from zope.component import adapts
from interfaces import IReadWriteFile
from zope.schema.interfaces import IField, IBool, IBytes, IText, IDate, IDatetime, IFloat, IInt
import datetime
import zope.datetime
from time import strptime
from zope.security.proxy import removeSecurityProxy

class FtpWidgetBase(object) :
    implements(IReadWriteFile)

    def __init__(self,field) :
        self.field = field

    @property
    def context(self) :
        return removeSecurityProxy(self.field).context


    def convertor(self,value) :
        return value 
        
    def serialize(self,value) :
        return str(value)        
        
    def write(self,value) :
        self.field.set(self.context,self.convertor(value))
        
    def read(self) :
        return self.serialize(self.field.get(self.context))
        
    def size(self) :
        return len(self.read())                
        
class FieldWidget(FtpWidgetBase) :
    adapts(IField)
    
    def convertor(self,value) :
        return eval(value)

class TextWidget(FtpWidgetBase) :
    adapts(IText)
    
class BytesWidget(FtpWidgetBase) :
    adapts(IBytes)

class DateTimeWidget(FtpWidgetBase) :
    adapts(IDatetime)
    fmt = "%Y-%m-%d %M:%H:%S"
    len = 19

    def convertor(self,value) :
        return datetime.datetime(*(strptime(value[0:self.len],self.fmt)[0:-2]+(zope.datetime.tzinfo(int(value[self.len:])),) ))
        
    def serialize(self,value) :
        return value.strftime(self.fmt+" %z")
        
class DateWidget(DateTimeWidget) :
    adapts(IDate)
    fmt = "%Y-%m-%d"




### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: attributedirectory.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"
 
import sys
from zope.interface import Interface,implements
from zope.app.container.interfaces import IReadContainer        
from zope.filerepresentation.interfaces import IReadDirectory
from zope.filerepresentation.interfaces import IReadFile
from interfaces import IAttribute, IAttributeDir
from zope.schema.interfaces import IField

class AttributeWrapper(object) :
    implements(IAttribute)
    def __init__(self,context,field) :
        self.context = context
        self.field = field

    def read(self) :
        return str(self.field.get(self.context.context.context))
        
        return str(getattr(
            self.context.context.context,
            self.attribute,
            None))

    def write(self,value) :
        return self.field.set(self.context.context.context,value)
        
        return setattr(
            self.context.context.context,
            self.attribute,
            value)

class AttributeDir2ReadDirectoryAdapter(dict):
    implements(IReadDirectory)

    def __init__(self,context) :
        self.context = context
        for iface in self.context.ifaces :
            for name in iface.names(all=True) :
                if name not in self :
                    if IField.providedBy(iface[name]) :
                        self[name] = iface[name].bind(self.context.context)
                        
class AttributeDirBase(object) :
    __name__ = "++at++"
    ifaces = []
    implements(IAttributeDir,IReadContainer)
    
    def __init__(self,context) :
        self.context = context    




### -*- coding: utf-8 -*- #############################################
"""Adapter of division to attribute dir component.

$Id: attributediradapter.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"


from ng.ftp.attributedirectory import AttributeDirBase
from zope.interface import Interface,implements,providedBy
from ng.ftp.interfaces import IAttributeDir

class AttributeDirAdapter(AttributeDirBase) :
    __doc__ = __doc__
    implements(IAttributeDir)
    
    @property
    def ifaces(self) :
        return list(providedBy(self.context))
    
    



### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: writeattribute.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"
 
from zope.interface import Interface,implements
from zope.filerepresentation.interfaces import IWriteFile


class Attribute2WriteFileAdapter(object) :
    implements(IWriteFile)
    
    def __init__(self,context) :
        self.context = context
        
    def write(self,value) :
        self.context.write(value)
        



### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: utils.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"

from pd.lib.utility import klass2name

def packkey(key,value) :
    return "=".join((key,klass2name(value.__class__)))
    
def unpackkey(key) :
    return key[0:key.rindex("=")]
    
def unpacktype(key) :
    return key[key.rindex("=")+1:]
    


### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: nsat.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"

from zope.interface import implements
from zope.traversing.interfaces import ITraversable
from zope.component.interface import nameToInterface
from zope.traversing.namespace import SimpleHandler
from zope.security.proxy import removeSecurityProxy
from zope.traversing.interfaces import IPhysicallyLocatable
from zope.location import location
from interfaces import IAttributeDir

class NsAt(SimpleHandler):
    """ ++at++"""
    implements(ITraversable)

    def __init__(self,*kv,**kw) :
        super(NsAt,self).__init__(*kv,**kw)

    def traverse(self,name,ignored) :
        return IAttributeDir(self.context)
        



### -*- coding: utf-8 -*- #############################################
#######################################################################
"""
Interfaces for the Zope 3 based ng.adapter.pager package

$Id: interfaces.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"

from zope.interface import Interface
from zope.schema import Field, Tuple, Int, Bool
from zope.filerepresentation.interfaces import IReadFile, IWriteFile


class IReadWriteFile(IReadFile, IWriteFile) :
    """ """
    pass




### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: readdirectory.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"
 
from zope.interface import Interface,implements
from zope.app.container.interfaces import IReadContainer        
from zope.filerepresentation.interfaces import IReadFile,IReadDirectory
from interfaces import IAttribute, IAttributeDir
from utils import packkey, unpackkey, unpacktype

class ReadDirectoryAdapter(object) :

    def __init__(self,context) :
        self.context = context
        
    def __getitem__(self,key) :
        """ A KeyError is raised if there is no value for the key."""
        if key == "++at++" :
            return IAttributeDir(self.context)
        return self.context[unicode(unpackkey(key))]

    def get(self,key, default=None) :
        """Get a value for a key.  The default is returned if there is no
           value for the key."""
        if key == "++at++" :
            return IAttributeDir(self.context)
        return self.context.get(unicode(unpackkey(key)), default=None)

    def keys(self) :
        """ Return the keys of the mapping object. """
        return [ x for x,y in self.items() ] + ["++at++"]

    def items(self) :
        """ Return the items of the mapping object. """
        return [ (packkey(x,y),y) for x,y in self.context.items()] + [('++at++',IAttributeDir(self.context))]
        
    def __contains__(self,key) :
        """ Tell if a key exists in the mapping.""" 
        key = unpackkey(key)
        return self.context.__contains__(unicode(key)) or key == '++at++'

    def __iter__(self) :
        """ Return an iterator for the keys of the mapping object. """
        
        yield "++at++"            
        for key,value in self.context.items() :
            yield packkey(key,value)
        
    def values(self) :
        """Return the values of the mapping object."""
        
        return list(self.context.value()) + [ IAttributeDir(self.context) ]

    def __len__(self) :
        return self.context.__len__() + 1




### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: directoryfactory.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"
 
from zope.interface import Interface,implements
from zope.filerepresentation.interfaces import IDirectoryFactory
from pd.lib.utility import name2klass
from ng.ftp.utils import packkey, unpackkey, unpacktype

class DirectoryFactory(object) :
    implements(IDirectoryFactory)
    def __init__(self,context) :
        self.context = context
        
    def __call__(self,name) :
        return name2klass(unpacktype(name))()



### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: readnotdirectory.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"
 
from zope.interface import Interface,implements
from zope.app.container.interfaces import IReadContainer        
from zope.filerepresentation.interfaces import IReadFile,IReadDirectory
from interfaces import IAttribute, IAttributeDir
from utils import packkey, unpackkey, unpacktype

class ReadNotDirectoryAdapter(object) :

    def __init__(self,context) :
        self.context = context
                
    def __getitem__(self,key) :
        """ A KeyError is raised if there is no value for the key."""
        if key == "++at++" :
            ir = IAttributeDir(self.context)
            return ir
        raise KeyError,key            

    def get(self,key, default=None) :
        """Get a value for a key.  The default is returned if there is no
           value for the key."""
        try :
            return self[key]
        except KeyError :
            return default                           

    def keys(self) :
        """ Return the keys of the mapping object. """
        return ["++at++"]

    def items(self) :
        """ Return the items of the mapping object. """
        return [('++at++',IAttributeDir(self.context))]
        
    def __contains__(self,key) :
        """ Tell if a key exists in the mapping.""" 
        return key == '++at++'

    def __iter__(self) :
        """ Return an iterator for the keys of the mapping object. """
        yield "++at++"            
        
    def values(self) :
        """Return the values of the mapping object."""
        return [ IAttributeDir(self.context) ]

    def __len__(self) :
        return 1




### -*- coding: utf-8 -*- #############################################
"""Container adapters the Zope 3 based contentobjects package

$Id: writedirectory.py 13543 2007-12-10 00:52:49Z cray $
"""
__author__  = "Andrey Orlov, 2007"
__license__ = "GPL"
__version__ = "$Revision: 13543 $"
 
from zope.interface import Interface,implements
from zope.filerepresentation.interfaces import IWriteFile,IWriteDirectory
from utils import packkey, unpackkey, unpacktype

class WriteDirectoryAdapter(object) :
    implements(IWriteDirectory)

    def __init__(self,context) :
        self.context = context
        
    def __setitem__(self,key,value) :
        """ A KeyError is raised if there is no value for the key."""
        self.context[unicode(unpackkey(key))] = value       

    def __delitem__(self,key,value) :
        del self.context[unicode(unpackkey(key))] 
